h1 {
  font-size: 28px !important;
  color: #a3a3a3 !important;
  text-transform: uppercase;
  letter-spacing: -2px;
}

.label-highlight {
  background: #f6fbfc !important;
  box-shadow: 0px 1px 0px rgba(0,0,0,0.15);
  padding: 0.2em 0.4em 0.2em 0.4em !important;
}

h3 .label-highlight {
  background: transparent !important;
  padding: 0.1em 0.4em 0px 0.4em !important;
  border-bottom: 1px solid #999;
  box-shadow: 0px 2px 1px rgba(0,0,0,0.15);
  border-radius: 0;
}

.comments .label-highlight {
  background: #fcfdf0 !important;
}

small.badge {
  font-size: 80% !important;
  font-weight: normal !important;
  display: inline-block;
  float: right;
}

form #form-options {
  color: #666;
  font-size: 95%;
  margin-top: 1.5em;
  padding: 0 0.25em;
}
form #form-options input {
  margin-top: 0.25em;
}

#facets .panel-heading {
  margin-bottom: 0;
}

.result {
  border-bottom: 1px solid #ccc;
  margin: 2em 0 0 0;
  padding: 0 0 1em 0;
}
.result:first-child {
  margin-top: 0.25em;
}

.result h3.title {
  font-size: 180% !important;
  font-family: 'Rokkitt', sans-serif;
  margin-top: 0;
}
.result h3.title small {
  font-size: 75% !important;
}

.result .body {
  font-family: Georgia, serif;
}

.result .category {
  font-family: 'Rokkitt', sans-serif;
}

.result .comments {
  color: #666666;
  font-size: 80%;
}
